//##protect##"FileHeader"
/** @file PCIe1016.cs
 *
 *  @authors S. Beuque, C. Guinot, J. Krauth, N. Reibel, J. Rose, L. Sartory, S. Weber
 *  @date    Creation: 01.02.2013
 *  @date    $LastChangedDate$
 *  @version $LastChangedRevision$
 *
 *  @par DESCRIPTION
 *  PCIe1016 .NET interface functions 
 *
 *  @par LICENCE
 *  @verbatim
 *  Copyright (C) 2009 - 2013 ADDI-DATA GmbH
 *
 *  ADDI-DATA GmbH
 *  Airpark Business Center
 *  Airport Boulevard B210
 *  77836 Rheinmnster
 *  Germany
 *  Tel: +49(0)7229/1847-0    |  Fax: +49(0)7229/1847-200
 *  http://www.addi-data-com  |  info@addi-data.com
 *  @endverbatim
 **/
//##protect##"FileHeader"

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCIe1016_NET
{
#region "User declarations"
//##protect##"UserDeclarations"
//##protect##"UserDeclarations"
#endregion

    public class PCIe1016
    {
        private int index;
        private IntPtr handle;

		private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

#region "Event Args and Handlers"
        public delegate void dummyEventHandler(object source);
        public event dummyEventHandler dummyEventReceived;

//##protect##"EventArgsAndHandlersDeclaration"
//##protect##"EventArgsAndHandlersDeclaration"
#endregion
#region "Constructor/Destructor"
        /// <summary>
        /// Class constructor
        /// </summary>
        private PCIe1016()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);

//##protect##"Constructor"
//##protect##"Constructor"
        }

        /// <summary>
        /// Class destructor
        /// </summary>
        ~PCIe1016()
        {
            DllImport.i_PCIe1016_CloseBoard(this.handle);
            this.index = -1;
            this.handle = IntPtr.Zero;

//##protect##"Destructor"
//##protect##"Destructor"
        }
#endregion
#region "Utils"
        /// <summary>
        /// Creates an instance of a PCIe1016 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCIe1016 on success</returns>
        public static PCIe1016 OpenBoard(int index)
        {
            PCIe1016 boardInstance;
            int nbBoard;
            IntPtr handle = IntPtr.Zero;

            // Get the number of boards
			nbBoard = GetBoardCount();
            if (nbBoard == -1)
            {
                Console.WriteLine("GetBoardCount failed!");
                return null;
            }

            // Check that the index is in the range
            if (index < 0 || index > nbBoard)
            {
                Console.WriteLine("Invalid index!");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCIe1016_OpenBoardViaIndex((Byte)index, out handle) != 0)
            {
                Console.WriteLine("i_PCIe1016_OpenBoardViaIndex failed!");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCIe1016();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Returns the number of PCIe1016 boards plugged in the computer
        /// </summary>
        /// <returns>The number of boards</returns>
        public static int GetBoardCount()
        {
            int ret;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCIe1016_GetNumberOfBoards(out nbBoard);
            if (ret != 0)
				return -1;

             // Success
            return (int)nbBoard;
        }

        /// <summary>
        /// Returns the description of the last error that occurred
        /// </summary>
        /// <returns>A string describing the last error that occurred</returns>
        public string GetLastError()
        {
            // Call the DLL function
            return Marshal.PtrToStringAnsi(DllImport.pc_PCIe1016_GetLastError(this.handle));
        }
#endregion
#region "Interrupt handling"
        /// <summary>
        /// Signals to all the listeners that an dummy event occurred.
        /// </summary>
        protected virtual void OndummyEventReceived()
		{
            if (this.dummyEventReceived != null)
                this.dummyEventReceived(this);
        }

        /// <summary>
        /// Interrupt function - Called each time the board has generated an interrupt
        /// </summary>
        private void InterruptFunction(IntPtr handle, Byte overflow, UInt32 source, UInt32 mask, IntPtr args, UInt32 argsSize)
        {
            // dummy ?
            if ((source & 0x0) != 0)
            {
                // Signal the event
                this.OndummyEventReceived();
            }

//##protect##"interruptFunctionBody"
//##protect##"interruptFunctionBody"
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1016_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCIe1016_ResetBoardIntRoutine(this.handle);
        }

#endregion

#region "PCIe1016"

#endregion
#region "DigitalInput"

		/// <summary>
		/// Reads 1 digital input of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="channel">Channel to be read</param>
		/// <param name="value">Status of the digital input</param>
		public int Read1DigitalInput(Byte channel, out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read1DigitalInput"
//##protect##"functionStart_Read1DigitalInput"
			returnValue = DllImport.i_PCIe1016_Read1DigitalInput(this.handle, channel, out value);
//##protect##"functionEnd_Read1DigitalInput"
//##protect##"functionEnd_Read1DigitalInput"
			return returnValue;
		}

		/// <summary>
		/// Reads 4 digital inputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port to be read. One port includes 4 channels</param>
		/// <param name="value">Status of the digital inputs</param>
		public int Read4DigitalInputs(Byte port, out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read4DigitalInputs"
//##protect##"functionStart_Read4DigitalInputs"
			returnValue = DllImport.i_PCIe1016_Read4DigitalInputs(this.handle, port, out value);
//##protect##"functionEnd_Read4DigitalInputs"
//##protect##"functionEnd_Read4DigitalInputs"
			return returnValue;
		}

		/// <summary>
		/// Reads 8 digital inputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="port">Port to be read. One port includes 8 channels</param>
		/// <param name="value">Status of the digital inputs</param>
		public int Read8DigitalInputs(Byte port, out Byte value)
		{
			int returnValue;
//##protect##"functionStart_Read8DigitalInputs"
//##protect##"functionStart_Read8DigitalInputs"
			returnValue = DllImport.i_PCIe1016_Read8DigitalInputs(this.handle, port, out value);
//##protect##"functionEnd_Read8DigitalInputs"
//##protect##"functionEnd_Read8DigitalInputs"
			return returnValue;
		}

		/// <summary>
		/// Reads the 16 digital inputs of the board
		/// </summary>
		/// <param name="deviceHandle">Handle of the board</param>
		/// <param name="value">Status of the digital inputs</param>
		public int Read16DigitalInputs(out UInt16 value)
		{
			int returnValue;
//##protect##"functionStart_Read16DigitalInputs"
//##protect##"functionStart_Read16DigitalInputs"
			returnValue = DllImport.i_PCIe1016_Read16DigitalInputs(this.handle, out value);
//##protect##"functionEnd_Read16DigitalInputs"
//##protect##"functionEnd_Read16DigitalInputs"
			return returnValue;
		}

#endregion
#region "User"
//##protect##"AssemblyUserCode"
//##protect##"AssemblyUserCode"
#endregion
    }
}
